#!/usr/bin/env python3

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches


def plot(fname,ax=None):
    if(ax==None):
        fig,ax = plt.subplots(figsize=(10,3))

    h=0.2
    total_angle=0

    parentk0={}
    labels=[]
    orbit = [[0,0]]
    dx = 0
    z0 = 0
    z1 = 0

    fd = open(fname,'r')
    for line in fd.readlines():
        line = line.strip()
        if line[0]=="#":
            continue
        line = line.split()
        keyw = line[0]
        name = line[1]
        z0   = float(line[2])
        d    = z0-z1
        z1   = float(line[3])
        params = [float(x) for x in line[4:]]

        l=z1-z0

        dx += d*np.sin(total_angle)
        orbit.append([z1-l/2,dx])

        if(keyw=='dipole'):
            rect = patches.Rectangle((z0,dx+-h/2),l,h,fc='w',ec='k')
            ax.add_patch(rect)
            if(name in parentk0):
                k0 = params[0] * parentk0[name]
            else:
                k0 = params[0]
                parentk0[name] = k0
            total_angle += np.arcsin(k0*l)

        if(keyw=='cavity'):
            lcell = 1
            n=int(l/lcell)
            #for z in np.linspace(z0,z1,n,endpoint=False):
            #    ell = patches.Ellipse((z,dx+0),1.5*l/n,h,fc='w',ec='k')
            #    ax.add_patch(ell)
            rect = patches.Rectangle((z0,dx+-h/2),l,h,fc='w',ec='k')
            ax.add_patch(rect)

        if(not name in labels):
            #sign = -1 if total_angle>0 else 1
            sign = 1
            ax.annotate(name.upper(),
                (z0,dx+sign*h*1.1/2 + (sign-1)/2*0.05),
                rotation=90,
                horizontalalignment='left', verticalalignment='bottom',
                rotation_mode='anchor'
            )
            labels.append(name)


    orbit.append([z1,dx])
    orbit=np.array(orbit).transpose()
    ax.set_xlim(0,z1)
    ax.plot(orbit[0],orbit[1],color='k',lw=2)
    fd.close()
    return ax



if(__name__=='__main__'):
    import sys
    ax = plot(sys.argv[1])
    plt.yticks([])
    #ax.set_xlabel("long. pos. [m]")
    if(len(sys.argv)>1):
        plt.savefig(sys.argv[2])
    else:
        plt.show()
